using UnityEngine;

public enum FPSWalkerState
{
    Idle,
    WalkFront,
    WalkBack,
    WalkLeft,
    WalkRight,
    Jump,
    WaterJump,
    Death
}

public class FPSWalker : MonoBehaviour
{
    public FloatVar healthPlayer;
    public AcidSea acidSea;
    public GUIText guiCheckpoint;
    public Chrono chrono;

    public TransformVar varSpawnPad;
    SpawnPad spawnPad;

	public float speed = 6.0f;
	public float jumpSpeed = 12.0f;
    public float gravity = 20.0f;

    public float airSpeed = 2.0f;

    public float seaJumpSpeed = 1.0f;
    public float seaGravity = 3.0f;

    public FPSWalkerState walkerState = FPSWalkerState.Idle;

    public Transform direction;

	Vector3 moveDirection = Vector3.zero;
	bool grounded = false;

    Quaternion originalRotation;

    void Start()
    {
        originalRotation = transform.localRotation;
    }

	void FixedUpdate()
	{
        if ( healthPlayer.Get() <= 0.0f )
        {
            walkerState = FPSWalkerState.Death;
            return;
        }

        Quaternion xQuaternion = Quaternion.AngleAxis( direction.rotation.eulerAngles.y, Vector3.up );
        transform.localRotation = originalRotation * xQuaternion;

        float keyH = Input.GetAxis( "Horizontal" );
        float keyV = Input.GetAxis( "Vertical" );

        // Correction for a constant speed even if moving in diagonal
        float coef = Mathf.Sqrt( Mathf.Pow( Mathf.Abs( keyH ) + Mathf.Abs(keyV), 2 ) );
        coef = (coef > 1.0f) ? (1 / coef) : 1.0f;
        
        keyH *= coef;
        keyV *= coef;

        if ( transform.position.y < (acidSea.transform.position.y - 1.5f) )
        {
            moveDirection.y = seaJumpSpeed + acidSea.GetSpeed();
            walkerState = FPSWalkerState.WaterJump;
        }
        else if ( grounded )
        {
            // We are grounded, so recalculate movedirection directly from axes
            moveDirection = new Vector3( keyH, 0, keyV );
            moveDirection = transform.TransformDirection( moveDirection );
            moveDirection *= speed;

            if ( Input.GetButton( "Jump" ) )
            {
                moveDirection.y = jumpSpeed;
                walkerState = FPSWalkerState.Jump;
            }
            else
            {
                if ( Mathf.Abs( keyH ) < 0.1f && Mathf.Abs( keyV ) < 0.1f )
                {
                    walkerState = FPSWalkerState.Idle;
                }
                else if ( Mathf.Abs( keyV ) >= Mathf.Abs( keyH ) )
                {
                    if ( keyV > 0.0F )
                    {
                        walkerState = FPSWalkerState.WalkFront;
                    }
                    else
                    {
                        walkerState = FPSWalkerState.WalkBack;
                    }
                }
                else
                {
                    if ( keyH > 0.0F )
                    {
                        walkerState = FPSWalkerState.WalkRight;
                    }
                    else
                    {
                        walkerState = FPSWalkerState.WalkLeft;
                    }
                }
            }
        }
        else
        {
            Vector3 airDirection = new Vector3( keyH, 0, keyV );
            airDirection = transform.TransformDirection( airDirection );
            airDirection *= airSpeed;

            if ( walkerState == FPSWalkerState.WaterJump )
            {
                // "air control" in water
                moveDirection.x = airDirection.x;
                moveDirection.z = airDirection.z;
            }
            else
            {
                // "air control" for jump
                //moveDirection += airDirection;
                //moveDirection.x = Mathf.Clamp( moveDirection.x, -speed, speed );
                //moveDirection.z = Mathf.Clamp( moveDirection.z, -speed, speed );
            }
        }
	
		// Apply gravity
        if ( walkerState == FPSWalkerState.WaterJump )
        {
            moveDirection.y -= (gravity / 5.0f) * Time.deltaTime;
        }
        else
        {
            moveDirection.y -= gravity * Time.deltaTime;
        }
		
		// Move the controller
		CharacterController controller = GetComponent<CharacterController>();
		CollisionFlags flags = controller.Move(moveDirection * Time.deltaTime);
		grounded = (flags & CollisionFlags.CollidedBelow) != 0;
	}

    void OnTriggerEnter ( Collider other )
    {
        SpawnPad newSpawnPad = other.GetComponent<SpawnPad>();

        if ( newSpawnPad )
        {
            // Unselect the previous spawnPad if there is one
            if ( spawnPad && spawnPad != newSpawnPad )
            {
                spawnPad.Unselect();
            }

            // Select the new spawnPad and register the time for the score
            newSpawnPad.Select();
            newSpawnPad.SetTime( Time.time - chrono.startTime );
            
            // Store the spawnPad position for the next spawn
            varSpawnPad.Set( newSpawnPad.transform );

            // Display the name of the spawnPad
            guiCheckpoint.text = newSpawnPad.name;

            spawnPad = newSpawnPad;
        }
    }
}

//@script RequireComponent(CharacterController)
