using UnityEngine;
using System.Collections;
using System.Collections.Generic;

public class MenuController : MonoBehaviour
{
    public GUISkin skin;
	private List<Animation> animationComponents;
	private List<AudioSource> audioSourceComponents;
	private float menuOn = 0;
	private float lastTime = 0;
	
	// Use this for initialization
	IEnumerator Start ()
    {
        //UpdateDifficulty();
		AudioListener.volume = 0;
		yield return 0;
		AudioListener.volume = 1;
	}
	
	// Update is called once per frame
	void Update ()
    {
		// Don't pause in first frame - allow scripts to settle in first
		if (Time.timeSinceLevelLoad == 0)
			return;
		
		float realDeltaTime = (Time.realtimeSinceStartup - lastTime);
		lastTime = Time.realtimeSinceStartup;
		menuOn = Mathf.Clamp01(menuOn + (Time.timeScale == 0 ? 1 : -1) * realDeltaTime * 5);
		
		if (!Screen.lockCursor && Time.timeScale != 0)
        {
			StartCoroutine(Pause(true));
		}
	}
	
	void OnGUI ()
    {
		if (menuOn == 0)
			return;
		
		GUI.skin = skin;
		
		// PLAY button
		Rect rect = new Rect(0, 0, 150, 50);
		rect.x = (Screen.width  - rect.width ) / 2 + (1 - menuOn) * Screen.width;
		rect.y = (Screen.height - rect.height) / 2 - 150;
		if (GUI.Button(rect, "PLAY")) 
        {
			StartCoroutine(Pause(false));
		}

        // Highscore text
        string highscoreText = "Highscore\n\n";
        Object[] objects = FindObjectsOfType( typeof( SpawnPad ) );
        foreach ( Object obj in objects )
        {
            SpawnPad spawnPad = (SpawnPad)obj;

            if ( spawnPad.name != "Start" )
            {
                highscoreText += spawnPad.name + "  ";
                if ( spawnPad.GetBestTime() < 0.0f )
                {
                    highscoreText += "---";
                }
                else
                {
                    highscoreText += spawnPad.GetBestTime().ToString( "f1" );
                }
                highscoreText += "\n\n";
            }
        }

        // Highscore box
        rect = new Rect( rect.x - 50, rect.y + 200, 250, 150 );
        GUI.Box( rect, highscoreText );
    }
	
	IEnumerator Pause (bool pause) 
    {
		// Pause/unpause time
		Time.timeScale = (pause ? 0 : 1);

        // Unlock/Lock cursor
		Screen.lockCursor = !pause;
		
		if (pause == true) 
        {
			Object[] objects = FindObjectsOfType(typeof(Animation));
			animationComponents = new List<Animation>();
			foreach (Object obj in objects) 
            {
				Animation anim = (Animation)obj;
				if (anim != null && anim.enabled) 
                {
					animationComponents.Add(anim);
					anim.enabled = false;
				}
			}
			objects = FindObjectsOfType(typeof(AudioSource));
			audioSourceComponents = new List<AudioSource>();
			foreach (Object obj in objects) 
            {
				AudioSource source = (AudioSource)obj;
				if (source != null && source.enabled /*&& source.isPlaying*/) 
                {
					audioSourceComponents.Add(source);
					source.Pause();
				}
			}
		}
		else 
        {
			// If unpausing, wait one frame before we enable animation component.
			// Procedural adjustments are one frame delayed because first frame
			// after being paused has deltaTime of 0.
			yield return 0;
			foreach (Animation anim in animationComponents)
				anim.enabled = true;
			foreach (AudioSource source in audioSourceComponents)
				source.Play();
			animationComponents = null;
		}
	}
}
