using UnityEngine;
using System.Collections;

public class MouseOrbit : MonoBehaviour
{
    public Transform target;
    public Transform yMinTransform;

    public float distance = 10.0F;

    public float xSpeed = 250.0F;
    public float ySpeed = 120.0F;

    public float yMinLimit = -20F;
    public float yMaxLimit = 80F;

    private float x = 0.0F;
    private float y = 0.0F;

    //@script AddComponentMenu("Camera-Control/Mouse Orbit")

    void Start()
    {
        Vector3 angles = transform.eulerAngles;
        x = angles.y;
        y = angles.x;

        // Make the rigid body not change rotation
        if ( rigidbody )
            rigidbody.freezeRotation = true;
    }

    void LateUpdate()
    {
        // Check if game is paused
        if ( Time.timeScale == 0 )
            return;

        if ( target )
        {
            Quaternion rotation;
            Vector3 position;
            float height;

            float deltaX = Input.GetAxis( "Mouse X" ) * xSpeed * 0.02F;
            float deltaY = Input.GetAxis( "Mouse Y" ) * ySpeed * 0.02F;

            x += deltaX;
            y -= deltaY;

            do
            {
                y = ClampAngle( y, yMinLimit, yMaxLimit );

                rotation = Quaternion.Euler( y, x, 0 );
                position = rotation * new Vector3( 0.0F, 0.0F, -distance ) + target.position;

                height = Terrain.activeTerrain.SampleHeight( position ) + 1.0F;
                height = Mathf.Max( height, (yMinTransform.position.y + 1.0f) );
                if ( position.y < height )
                {
                    y += 1.0F;
                }
            }
            while ( position.y < height && y < yMaxLimit );

            transform.rotation = rotation;
            transform.position = position;
        }
    }

    float ClampAngle( float angle, float min, float max )
    {
        if ( angle < -360 )
            angle += 360;
        if ( angle > 360 )
            angle -= 360;
        return Mathf.Clamp( angle, min, max );
    }
}